#include "globals.h"
#include "usart.h"


ISR(USART_RXC_vect)
{
	cli();
	if(!p_Flag.bRec)	//1.Byte empfangen
	{
		p_cRecReg[0] = UDR;
		p_iSZaehler = 1;
		p_Flag.bRec = 1;
	}
	else if(p_Flag.bRec)	//fr 2. und 3.Byte
	{
		p_cRecReg[p_iSZaehler] = UDR;
		p_iSZaehler++;
		if(p_iSZaehler == 3)
		{
			EvalRecData();
			p_Flag.bRec = 0;
		}
	}
	sei();
}

ISR(USART_UDRE_vect)
{
	cli();
	//Paritt bertragen
	if(p_Flag.bParityFull)
	{
		UDR = p_cParity;
		p_cParity = 0;
		p_Flag.bParityFull = 0;
	}
	
	if(p_Flag.bEnd)		//wenn bertragung beendet, Interrupts deaktivieren
	{
		UCSRB &= ~ ((1<<TXCIE)|(1<<UDRIE));
		UCSRB |= (1<<RXCIE);
		p_Flag.bEnd = 0;
	}
	
	//Senden CPU Fequenz
	else if(p_cStatus == STA_SCPU)
	{
		if(p_iRZaehler ==1 )
		{
			UDR = 4;
		}
		else
		{
			SendUDR(p_iRZaehler);
		}
		p_iRZaehler++;
		
		if(p_iRZaehler == 6)
		{
			p_iRZaehler = 1;
			p_Flag.bParityFull = 1;
			p_Flag.bEnd = 1;
		}
	}
	//Senden ADC Teiler
	else if(p_cStatus == STA_SADCT)
	{
		if(p_iRZaehler == 1)
		{
			UDR = 2;
			p_iRZaehler++;
		}
		else if(p_iRZaehler == 2)
		{
			SendUDR( ADCSRA & 0b000000111);
			p_iRZaehler = 1;
			p_Flag.bParityFull = 1;
			p_Flag.bEnd = 1;
		}		
	}
	else if(p_cStatus == STA_SDATA)	//Sende Datenstze
	{
		if(p_iRZaehler == 1)
			UDR = sDataCounter / 256;
		else if(p_iRZaehler==2)
			UDR = sDataCounter % 256;
		else
		{
			SendUDR(caData[p_iRZaehler-3]);
			
			if(p_iRZaehler>sDataCounter+1)
			{
				p_iRZaehler =1;
				p_Flag.bParityFull = 1;
				p_Flag.bEnd = 1;
			}
		}
		p_iRZaehler++;
	}
	else	//sende Zeichensatz fr bertragung komplett
	{
		if(p_iRZaehler == 1)
			UDR = 0;
		if(p_iRZaehler == 2)
			UDR = 1;
		if(p_iRZaehler == 3)
		{
			UDR = 11;
			p_Flag.bEnd = 1;
		}
		p_iRZaehler++;
	}
	sei();
}

void SendUDR(unsigned char cData)	//fr zu bertragendes Byte zur Paritt hinzu und sendet Byte ber UART
{
	UDR = cData;
	p_CountParity++;
	p_cParity ^= cData;	
	
	if(p_CountParity == 8)
	{
		p_Flag.bParityFull = 1;
		p_CountParity = 0;
	}
}

void init(void)	//Settings fr UART setzen
{
	p_Flag.bParityFull = 0;
	p_Flag.bEnd = 0;
	p_Flag.bRec = 0;
	p_CountParity = 0;
	p_cParity = 0;
	UBRRL = 71;
	UCSRB |= (1<<RXEN | 1<<TXEN | 1<<RXCIE);
	sei();	
}

void EvalRecData(void)		//Werte die empfangenen Daten aus
{
	if(p_cRecReg[0] == ART_COM)
	{
		if(p_cRecReg[2] == COM_GETDATA)
		{
			p_cStatus = STA_SCPU;
			UDR = 1;
			UCSRB |= ((1<<TXCIE)|(1<<UDRIE));
			UCSRB &= ~(1<<RXCIE);
		}
		else 
		{
			if(p_cRecReg[2] == COM_LDATAOK)
				p_cStatus++;
			if(p_cStatus <= STA_SDATA)
				UDR = p_cStatus;
			UCSRB |= ((1<<TXCIE)|(1<<UDRIE));
			UCSRB &= ~(1<<RXCIE);
		}
		p_iRZaehler = 1;

	}
}
